@@ -1,4 +1,4 @@
-/* Copyright (C) 2000  Britton Leo Kerin, see copyright. */
+/* Copyright (C) 2006  Britton Leo Kerin, see copyright. */
 
 /* This function performs the actual playing run as specified by the
    options. */
@@ -300,7 +300,7 @@ void play(parameters_stt *clp) /* pneumonic: app: actual params pointer */
     if ( (rtn = pthread_attr_setschedpolicy(&move_fd_attr, SCHED_FIFO)) )
       err_die("BUG: pthread_attr_setschedpolicy failed: %s\n", strerror(rtn));
     move_fd_param.sched_priority = fifo_max_prio;
-    if ( (rtn = pthread_attr_setschedparam(&move_fd_attr, &move_au_param)) )
+    if ( (rtn = pthread_attr_setschedparam(&move_fd_attr, &move_fd_param)) )
       err_die("BUG: pthread_attr_setschedparam failed: %s\n", strerror(rtn));
     if ( (rtn = pthread_attr_setscope(&move_fd_attr, PTHREAD_SCOPE_SYSTEM)) )
       err_die("BUG: pthread_attr_setscope failed: %s\n", strerror(rtn));
@@ -355,16 +355,18 @@ void play(parameters_stt *clp) /* pneumonic: app: actual params pointer */
   }
 
   /* Start threads.  If we have root authority, then these threads
-     will be using real time scheduling, and we must therefore be
-     root to start them, and they therefore inherit effective root
-     ids, but drop them immediately on entry. */  
-  if ( (rtn = pthread_create(&move_au_th, &move_au_attr, 
-			     (void *(*)(void *)) move_au, &au_th_arg)) ) {
-    err_die("BUG: pthread_create failed: %s\n", strerror(rtn));
+     will be using real time scheduling, and we must therefore be root
+     to start them, and they therefore inherit effective root ids, but
+     drop them as soon as all threads have been created.. */  
+  if ( (rtn = pthread_create (&move_au_th, &move_au_attr, 
+			      (void *(*)(void *)) move_au, &au_th_arg)) ) {
+    err_die ("BUG: pthread_create failed to create audio thread: %s\n",
+	     strerror (rtn));
   }
-  if ( (rtn = pthread_create(&move_fd_th, &move_fd_attr, 
-			     (void *(*)(void *)) move_fd, &fd_th_arg)) ) {
-    err_die("BUG: pthread_create failed: %s\n", strerror(rtn));
+  if ( (rtn = pthread_create (&move_fd_th, &move_fd_attr, 
+			      (void *(*)(void *)) move_fd, &fd_th_arg)) ) {
+    err_die ("BUG: pthread_create failed to create file thread: %s\n",
+	     strerror (rtn));
   }
 
   if ( have_root_authority ) {
@@ -372,7 +374,13 @@ void play(parameters_stt *clp) /* pneumonic: app: actual params pointer */
     if ( seteuid(getuid()) == -1 )
       /* If for some crazy reason we fail to drop root permissions, exit 
 	 immediately without advertising the fact. */
-      err_die("BUG: internal error\n");
+      exit (EXIT_FAILURE);
+    /* Let other threads know that we have dropped root permissions
+       (they wait for this before doing anything).  */
+    pthread_mutex_lock(&root_permissions_dropped_mutex);
+    root_permissions_dropped = 1;
+    pthread_cond_broadcast(&root_permissions_dropped_cv);
+    pthread_mutex_unlock(&root_permissions_dropped_mutex);
   }
 
   /* Restore the default mask.  Note that at no time has the default
