@@ -1,17 +1,15 @@
-/* Copyright (C) 2000  Britton Leo Kerin, see copyright. */
+/* Copyright (C) 2006  Britton Leo Kerin, see copyright. */
 
 /* This function performs the actual recording run as specified by the
    options. */
 
-#include <assert.h>/***/
-#include <stdio.h>/***/
-
 #include <errno.h>
 #include <math.h>
 #include <sched.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
+#include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <sys/mman.h>
@@ -280,14 +278,16 @@ void record(parameters_stt *clp)
   /* Start threads.  If we have root authority, then these threads
      will be using real time scheduling, and we must therefore be
      root to start them, and they therefore inherit effective root
-     ids, but drop them immediately on entry. */  
-  if ( (rtn = pthread_create(&move_au_th, &move_au_attr, 
+     ids, but drop them as soon as all threads have been created. */  
+  if ( (rtn = pthread_create (&move_au_th, &move_au_attr, 
 			     (void *(*)(void *)) move_au, &au_th_arg)) ) {
-    err_die("BUG: pthread_create failed: %s\n", strerror(rtn));
+    err_die ("BUG: pthread_create failed to create audio thread: %s\n",
+	     strerror (rtn));
   }
-  if ( (rtn = pthread_create(&move_fd_th, &move_fd_attr, 
-			     (void *(*)(void *)) move_fd, &fd_th_arg)) ) {
-    err_die("BUG: pthread_create failed: %s\n", strerror(rtn));
+  if ( (rtn = pthread_create (&move_fd_th, &move_fd_attr, 
+			      (void *(*)(void *)) move_fd, &fd_th_arg)) ) {
+    err_die ("BUG: pthread_create failed to create file thread: %s\n",
+	     strerror (rtn));
   }
 
   if ( have_root_authority ) {
@@ -296,6 +296,12 @@ void record(parameters_stt *clp)
       /* If for some crazy reason we fail to drop root permissions, exit
 	 immediately without advertising the fact. */
       err_die("BUG: internal error\n");
+    /* Let other threads know that we have dropped root permissions
+       (they wait for this before doing anything).  */
+    pthread_mutex_lock(&root_permissions_dropped_mutex);
+    root_permissions_dropped = 1;
+    pthread_cond_broadcast(&root_permissions_dropped_cv);
+    pthread_mutex_unlock(&root_permissions_dropped_mutex);
   }
 
   /* Restore the default mask.  Note that at no time has the default
